local frame = CreateFrame("Frame"); -- Need a frame to respond to events
EVENTS_TO_REGISTER = {"ADDON_LOADED", "COMBAT_LOG_EVENT_UNFILTERED", "PLAYER_ENTERING_WORLD", "PLAYER_LOGOUT", "CHAT_MSG_ADDON", "ENCOUNTER_START", "ENCOUNTER_END", "PLAYER_REGEN_DISABLED", "PLAYER_REGEN_ENABLED"}
for _, v in pairs(EVENTS_TO_REGISTER) do
	frame:RegisterEvent(v)
end


-- Addon
local hf = CreateFrame("Frame",modName,UIParent);
loaded = false


_G.HardFinger = {}
local HardFinger = _G.HardFinger
local playerName = UnitName("player")
HardFinger.HardFinger_Config = HardFinger.HardFinger_Config or {}
local P = HardFinger.HardFinger_Config

COMBATLOG_OBJECT_TYPE_PET		= COMBATLOG_OBJECT_TYPE_PET or 0x00001000

COMBATLOG_OBJECT_TYPE_GUARDIAN		= COMBATLOG_OBJECT_TYPE_GUARDIAN or 0x00002000

COMBATLOG_OBJECT_TYPE_NPC 		= COMBATLOG_OBJECT_TYPE_NPC or 0x0000800

PET_FILTER_BITMASK			= COMBATLOG_OBJECT_TYPE_PET + COMBATLOG_OBJECT_TYPE_GUARDIAN + COMBATLOG_OBJECT_TYPE_NPC

COMBATLOG_OBJECT_REACTION_FRIENDLY	= COMBATLOG_OBJECT_REACTION_FRIENDLY or 0x00000010


function HardFinger.UpdateGUIVariables()
	print("HardFinger - GUI Options Updated")

	HardFinger.ListOfChannels = {}
	if IsInGuild() then
			HardFinger.ListOfChannels["/g GuildChat"] = "GUILD_CHANNEL"
	end
	for i=5,15,1 do
		a,b = GetChannelName(i)
		if a ~= 0 and b ~= 0 then
			HardFinger.ListOfChannels[a .. ". " .. b] = a
		end
	end

--	list = { ["Name only"] = "normal", ["Use player titles"] = "title", ["Copy from original tip"] = "original", ["Mary Sue Protocol"] = "marysueprot" }
end

function HardFinger.ForceLoad(updateOnly)
	if updateOnly ~= true then
		HFDB = HFDB or { }

		HardFinger.HardFinger_Config.Profiles = HFDB

		HFDB.Default = { }
		HFDB.Default.Name = "Default"
		HardFinger.DefaultVariables = {"Status", "WhoPulled", "Taunt", "BloodLust", "BattleRez", "TimeToDie", "PrivateChannel"}
		for _, v in pairs(HardFinger.DefaultVariables) do
			HFDB.Default[v] = false
		end

		-- Does the player has a valid active Profile selected?
		if HFDB[playerName] == nil then
			HFDB[playerName] = HFDB.Default
		end
		loaded = true	
	end


	for _, v in pairs(HardFinger.DefaultVariables) do
		HardFinger.HardFinger_Config[v] = HFDB[playerName][v]
	end

	if (HardFingerOptions:IsShown()) then
		HardFingerOptions:Hide();
		HardFingerOptions:Show();
	end
end


function HardFinger.SaveVariables()
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = HardFinger.HardFinger_Config[v]
	end
	HardFinger.ForceLoad(true)
end

function HardFinger.ResetProfile()
	print("HardFinger - Profile: "..playerName.." reseted.")
	for _, v in pairs(HardFinger.DefaultVariables) do
		HFDB[playerName][v] = false
	end
	HardFinger.SaveVariables()
end


--
local function OnEvent(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		print("Entering World...")
		playerName = UnitName("player")
		HardFinger.CombatStatus = UnitAffectingCombat("player")
		HardFinger.WorldInfo()

	elseif event == "PLAYER_LOGOUT" or event == "PLAYER_LOGIN" then
		HardFinger.SaveVariables()
	elseif event == "ADDON_LOADED" then
		HardFinger.ForceLoad()

	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" and P.Status == true then
		_, message, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellID, spellName, param12, param13, param14, param15, _, _, param18 = ...

		-- Always running Display failTable time
		if HardFinger.FailTableDelayTimer ~= nil then
        HardFinger.DisplayFailTable()
    end


		if message == "SPELL_SUMMON" then

			if bit.band(destFlags, PET_FILTER_BITMASK) ~= 0 then
				HardFinger.PetOwnersTable[destGUID] = {sourceName, sourceGUID}

			end
		end

		if P.WhoPulled == true then
			HardFinger.WhoPulled("ALL", HardFinger.DBMPullTimer, message, sourceGUID, sourceName, destGUID, destName, destFlags, spellID)
		end
		if P.Taunt == true then
			HardFinger.WhoTaunted(message, sourceName, destName, destRaidFlags, spellID)		
		end
		if P.BattleRez == true then
			HardFinger.WhoRezzed(message, wsourceName, destName, destRaidFlags, spellID)
		end
		if P.BloodLust == true then
			HardFinger.WhoLusted()
		end			
	elseif event == "PLAYER_REGEN_ENABLED" then
		HardFinger.CombatStatus = false
		HardFinger.CombatToggle("OUT_OF_COMBAT")
	elseif event == "PLAYER_REGEN_DISABLED" then
		HardFinger.CombatStatus = true
		HardFinger.CombatToggle("IN_COMBAT")
	elseif event == "ENCOUNTER_START" then
		param1 = ...
	        HardFinger.ActiveEncounter = param1
		HardFinger.timestampDisplayTableTimed = nil

		print(event, encounterID, ...)

		HardFinger.DisplayFailTable("HRESET")

		HardFinger.WorldInfo()
	elseif event == "ENCOUNTER_END" then
	        HardFinger.LastEncounter = HardFinger.ActiveEncounter
		HardFinger.ActiveEncounter = 0
		print(event, encounterID, ...)
		print("Requesting to display FailTable")
		HardFinger.DisplayFailTable("RESET")
		HardFinger.CombatStatus = false
	elseif event == "CHAT_MSG_ADDON" then
		_, prefix, message = ...
		if prefix == "D4" and string.find(message"PT") then
		
        local index = string.find(param2, "\t", 4) - 1

			local timer = string.sub(param2, 4, index)
			HardFinger.DBMPullTimer = timer+GetTime()
    
		end
	end
end

frame:SetScript("OnEvent", OnEvent)
SLASH_HARDFINGER1 = "/hf"
function SlashCmdList.HARDFINGER(msg, editbox)
	msg = string.lower(msg)
	if msg == "" then
		print("\t Type...")
		print("\t|cffff0000/hf stop |r - To pause the addon")
		print("\t|cff00ff00/hf resume |r - To resume the addon")

		local loaded, reason = true
		if (loaded) == true then
			if (HardFingerOptions:IsShown()) then
				HardFingerOptions:Hide();
			else
				HardFinger.UpdateGUIVariables();
				HardFingerOptions:Show();
			end
		else
			AzMsg("Could not open HardFinger Options: |1"..tostring(reason).."|r. Please make sure the addon is enabled from the character selection screen.");
		end
	end
	if msg == "reset" then
		HardFinger.ResetProfile()
	elseif msg == "wp" or msg == "whopulled" then
		P.WhoPulled = HardFinger.toggletf(P.WhoPulled)
		print("HardFinger - WhoPulled is "..HardFinger.StatusText(WhoPulled))
	elseif msg == "bloodlust" or msg == "lust" or msg == "bl" then
		P.BloodLust = HardFinger.toggletf(P.BloodLust)
		print("HardFinger - BloodLust is "..HardFinger.StatusText(BloodLust))
	elseif msg == "ttd" or msg == "time to die" then
		P.TimeToDie = HardFinger.toggletf(P.TimeToDie)
		print("HardFinger - TimeToDie is "..HardFinger.StatusText(TimeToDie))
	elseif msg == "battle rez" or msg == "battlerez" then
		P.BattleRez = HardFinger.toggletf(P.BattleRez)
		print("HardFinger - BattleRez is "..HardFinger.StatusText(BattleRez))
	elseif msg == "taunt" then
		P.Taunt = HardFinger.toggletf(P.Taunt)
		print("HardFinger - Taunt is "..HardFinger.StatusText(Taunt))
	elseif msg == "stop" then
		print("HardFinger - Stopped.")
		P.Status = false
		frame:UnregisterAllEvents()
	elseif msg == "fl" then
		HardFinger.ForceLoad()
		print("Hardfinger - Load forced")
	elseif msg == "sv" then
		HardFinger.SaveVariables()
		print("Hardfinger - Saving Variables")	
	elseif msg == "dft" then
		print("Hardfinger - Displaying Fail Table")
		HardFinger.DisplayFailTable()
	elseif msg == "wft" then
		print("Hardfinger - Wiping Fail Table")
		wipe(HardFinger.FailTable)
	elseif msg == "d" then
		print("HardFinger - Status Debugg")
		for _, v in pairs(DefaultVariables) do
			dbgt = P[v]
			if dbgt == nil then dbgt = "nil" end
			print(v..": "..tostring(dbgt))
		end
	elseif msg == "resume" then
		print("HardFinger - Resumed.")
		P.Status = true
		for _, v in pairs(EVENTS_TO_REGISTER) do
			frame:RegisterEvent(v)
		end
	end
end

-- Variables Loaded
function hf:VARIABLES_LOADED(event)
	isColorBlind = (GetCVar("colorblindMode") == "1");
	-- Default Fonts
	TT_DefaultConfig.fontFace = GameFontNormal:GetFont();
	TT_DefaultConfig.barFontFace = NumberFontNormal:GetFont();
	-- Init Config
	if (not TipTac_Config) then
		TipTac_Config = {};
	end
	cfg = setmetatable(TipTac_Config,{ __index = TT_DefaultConfig });
	-- Default the bar texture if it no longer exists
	bars[1]:SetStatusBarTexture(cfg.barTexture);
	if (not bars[1]:GetStatusBarTexture()) then
		cfg.barTexture = nil;
	end
	-- Hook Tips & Apply Settings
	self:HookTips();
	self:ApplySettings();
	-- Position
	if (cfg.left and cfg.top) then
		self:ClearAllPoints();
		self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",cfg.left,cfg.top);
	else
		self:Show();
		-- Just set left and top here, in case the player just closes the anchor without moving, so it doesn't keep showing up when they log in
		cfg.left, cfg.top = self:GetLeft(), self:GetTop();
	end
	-- Cleanup
	self:UnregisterEvent(event);
	self[event] = nil;
end